/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef demcmds_h
#define demcmds_h

#ifdef VTK

#include <Unidraw/Commands/command.h>
#include <MapUnidraw/mapimport.h>
#include <vtkLookupTable.h>
#include <vtkRibbonFilter.h>

class OpenFileChooser;

class WaterLandLookupTable : public vtkLookupTable {
public:
  static WaterLandLookupTable *New() {return new WaterLandLookupTable;};
    WaterLandLookupTable();
    virtual unsigned char* MapValue(float v);
  virtual void Build() {}; // prevent construction of linear ramp that
                           // screws up this simple direct insertion table
};

class RoadRibbonFilter : public vtkRibbonFilter {
public:
  static RoadRibbonFilter *New() {return new RoadRibbonFilter;};
    RoadRibbonFilter();
    char *GetClassName() {return "RoadRibbonFilter";}
protected:
    void Execute();
};

class RibbonFixFilter : public vtkPolyDataToPolyDataFilter {
public:
  static RibbonFixFilter *New() {return new RibbonFixFilter;};
    RibbonFixFilter();
    char *GetClassName() {return "RibbonFixFilter";}
protected:
    void Execute();
};

class CameraCmd : public Command {
public:
    CameraCmd(float px, float py, float fx, float fy, boolean project=true);
    virtual void Execute();
    virtual boolean Reversible() { return false; }
protected:
    float px_, py_, fx_, fy_;
    boolean project_;
};

class DemCmd : public Command {
public:
  DemCmd(ControlInfo*);
  virtual Command* Copy();
protected:
  void get_camera (float& px, float& py, float& pz,
		   float& fx, float& fy, float& fz);
};

class DemCameraCmd : public DemCmd {
public:
  DemCameraCmd(ControlInfo*);
  virtual Command* Copy();
  void set_camera (float px, float py, float pz, float fx, float fy, float fz);
protected:
  void yaw_camera (float ang);
  void update_camera (float px, float py, float fx, float fy, boolean project);
  void reset_camera();
  void update_states();
};

class DemLoadDemCmd : public DemCameraCmd {
public:
    DemLoadDemCmd(ControlInfo*);
    DemLoadDemCmd(const char*);
    virtual ~DemLoadDemCmd();
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
    OpenFileChooser* chooser_;
    char* fname_;
};

class DemLoadLODDemCmd : public Command {
public:
    DemLoadLODDemCmd(ControlInfo*);
    virtual ~DemLoadLODDemCmd();
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
};

class DemImportFCCmd : public MapImportCmd {
public:
    DemImportFCCmd(ControlInfo* =nil, ImportChooser* =nil);
    virtual void Execute();
};

class DemLoadFCCmd : public Command {
public:
    DemLoadFCCmd(ControlInfo* =nil);
    DemLoadFCCmd(const char*);
    virtual void Execute();
protected:
    OpenFileChooser* chooser_;
    char* fname_;
};

class DemExportVRMLCmd : public Command {
public:
    DemExportVRMLCmd(ControlInfo*);
    virtual ~DemExportVRMLCmd();
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
    OpenFileChooser* chooser_;
};

class DemInitCmd : public Command {
public:
    DemInitCmd(ControlInfo*);
    virtual ~DemInitCmd();
    virtual void Execute();
    virtual Command* Copy();
};

class DemCameraResetCmd : public DemCameraCmd {
public:
  DemCameraResetCmd(ControlInfo* ci);
    virtual ~DemCameraResetCmd();
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
};

class DemCameraUpdateCmd : public DemCameraCmd {
public:
  DemCameraUpdateCmd(ControlInfo* ci, float px, float py, float fx, float fy,
		     boolean project =true);
    virtual ~DemCameraUpdateCmd();
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
  float _px, _py, _fx, _fy;
  boolean _project;
};

class DemCameraUpCmd : public DemCameraCmd {
public:
    DemCameraUpCmd(ControlInfo* =nil, float up_amount =30.0);
    virtual ~DemCameraUpCmd();
    virtual void Execute();
    virtual boolean Reversible();
    float GetUpAmt() { return upamt; }
    void SetUpAmt(float amt) { upamt = amt; }
    virtual Command* Copy();
protected:
    float upamt;
};

class DemCameraDownCmd : public DemCameraCmd {
public:
    DemCameraDownCmd(ControlInfo* =nil, float dn_amount =30.0);
    virtual ~DemCameraDownCmd();
    virtual void Execute();
    virtual boolean Reversible();
    float GetDownAmt() { return dnamt; }
    void SetDownAmt(float amt) { dnamt = amt; }
    virtual Command* Copy();
protected:
    float dnamt;
};

class DemCameraYawLeftCmd : public DemCameraCmd {
public:
    DemCameraYawLeftCmd(ControlInfo* =nil, float yl_amount =15.0);
    virtual ~DemCameraYawLeftCmd();
    virtual void Execute();
    virtual boolean Reversible();
    float GetYawAmt() { return ylamt; }
    void SetYawAmt(float amt) { ylamt = amt; }
    virtual Command* Copy();
protected:
    float ylamt;
};

class DemCameraYawRightCmd : public DemCameraCmd {
public:
    DemCameraYawRightCmd(ControlInfo* =nil, float yr_amount =15.0);
    virtual ~DemCameraYawRightCmd();
    virtual void Execute();
    virtual boolean Reversible();
    float GetYawAmt() { return yramt; }
    void SetYawAmt(float amt) { yramt = amt; }
    virtual Command* Copy();
protected:
    float yramt;
};

class DemCameraDollyCmd : public DemCameraCmd {
public:
    DemCameraDollyCmd(ControlInfo* =nil, float amount = 0.9);
    virtual void Execute();
    virtual boolean Reversible();
    virtual Command* Copy();
protected:
    float _amt;
};

class DemShowCamStateCmd : public Command {
public:
  DemShowCamStateCmd(ControlInfo* =nil);
  virtual void Execute();
};

class DemHideCamStateCmd : public Command {
public:
  DemHideCamStateCmd(ControlInfo* =nil);
  virtual void Execute();
};

class DemNoCullCmd : public Command {
public:
  DemNoCullCmd(ControlInfo* =nil);
  virtual void Execute();
};

class Dem2DimFootprintCullCmd : public Command {
public:
  Dem2DimFootprintCullCmd(ControlInfo* =nil);
  virtual void Execute();
};

class DemRepresentationCmd : public Command {
public:
    enum DemRep { POINTS, WIREFRAME, SURFACE };
    DemRepresentationCmd(ControlInfo* = nil, DemRep rep =WIREFRAME);
    virtual boolean Reversible();
    virtual void Execute();
    virtual Command* Copy();
protected:
    DemRep _rep;
};

class DemShadingCmd : public Command {
public:
    enum DemShade { FLAT, GOURAUD, PHONG };
    DemShadingCmd(ControlInfo* = nil, DemShade shade =FLAT);
    virtual boolean Reversible();
    virtual void Execute();
    virtual Command* Copy();
protected:
    DemShade _shade;
};

class DemBackfaceCullCmd : public Command {
public:
    DemBackfaceCullCmd(ControlInfo* = nil, boolean bfc =false);
    virtual boolean Reversible();
    virtual void Execute();
    virtual Command* Copy();
protected:
    boolean _bfc;
};

class DemStartCaptureCmd : public Command {
public:
    DemStartCaptureCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemStopCaptureCmd : public Command {
public:
    DemStopCaptureCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleInitCmd : public DemCmd {
public:
    DemVehicleInitCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleForwardCmd : public Command {
public:
    DemVehicleForwardCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleBackwardCmd : public Command {
public:
    DemVehicleBackwardCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleTurnLeftCmd : public Command {
public:
    DemVehicleTurnLeftCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleTurnRightCmd : public Command {
public:
    DemVehicleTurnRightCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemToggleRteCaptureCmd : public Command {
public:
    DemToggleRteCaptureCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemVehicleGasCmd : public Command {
public:
    DemVehicleGasCmd(ControlInfo* = nil);
    virtual void Execute();

    static float _speed;
    static float _incr;
};

class DemVehicleBrakeCmd : public Command {
public:
    DemVehicleBrakeCmd(ControlInfo* = nil);
    virtual void Execute();
};

class DemNewViewCmd : public Command {
public:
  DemNewViewCmd(ControlInfo* =nil);
  virtual void Execute();
  virtual Command* Copy();
};

class DemMoveActorCmd : public Command {
public:
  DemMoveActorCmd(Editor*, char* name, float x, float y, float xdir, float ydir);
  virtual void Execute();
protected:
  char* _name;
  float _x, _y, _xdir, _ydir;
};

#endif

#endif
