/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapprim.h>

#include <Dlg/dlglib.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlgfeature.h>

#include <MapUnidraw/mapcomps.h>

#include <DlgUnidraw/dlgclasses.h>
#include <DlgUnidraw/dlgcomps.h>
#include <DlgUnidraw/dlgeditor.h>
#include <DlgUnidraw/dlgscripts.h>
#include <DlgUnidraw/dlgviewstate.h>

#include <OverlayUnidraw/ovpage.h>
#include <OverlayUnidraw/ovviewer.h>
#include <OverlayUnidraw/paramlist.h>

#include <Unidraw/Commands/command.h>

#include <Unidraw/ulist.h>

#include <Unidraw/unidraw.h>

#include <InterViews/cursor.h>
#include <InterViews/window.h>


/*****************************************************************************/

ParamList* DlgFClassComp::_dlg_fcl_params = nil;

DlgFClassComp::DlgFClassComp(DlgFeatureClass* mfc) : MapFClassComp(mfc) {
}

DlgFClassComp::DlgFClassComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* DlgFClassComp::GetParamList() {
    if (!_dlg_fcl_params) 
	GrowParamList(_dlg_fcl_params = new ParamList());
    return _dlg_fcl_params;
}

void DlgFClassComp::GrowParamList(ParamList* pl) {
    pl->add_param("fclass", ParamStruct::required, &DlgFClassScript::ReadName, this, &_mapfc);
    MapFClassComp::GrowParamList(pl);
}

DlgFClassComp::~DlgFClassComp() {
}

void DlgFClassComp::Interpret(Command* cmd) {
    if (cmd->IsA(MAPSHOW_CMD) || cmd->IsA(MAPSHOWALL_CMD) && _comps->IsEmpty()) {
      DlgViewState* viewstate =nil;
	DlgEditor* editor = (DlgEditor*)cmd->GetEditor();
	if (editor) {
	editor->GetWindow()->cursor(hourglass);

	if (editor->Initializing()) {
	    DlgViewState* view = new DlgViewState((DlgFeatureClass*)_mapfc);
	    editor->SetMapViewState(view);
	    double x1, y1, x2, y2;
	    int zone;
	    ((DlgFeatureClass*)_mapfc)->extent(x1, y1, x2, y2, zone);
	    editor->GetViewer()->SetPage(new OverlayPage((x2-x1+1)/2000.0,
							 (y2-y1+1)/2000.0));
	    editor->Initializing(false);
	}
	editor->SetProjectionLocal(((DlgFeatureClass*)_mapfc)->projection());
	viewstate = (DlgViewState*)editor->GetMapViewState();
	}
	for (int i = 0; i < _mapfc->nfeats(); i++) {
	    MapFeature* feature = _mapfc->feature(i);
	    if (feature->type() != MapFeature::JunctionType && feature->long_attr("MAJOR_CODE") == 0) continue;
	    if (feature->type() != MapFeature::JunctionType && feature->long_attr("MAJOR_CODE") != 40) continue;
	    if (feature->type() != MapFeature::JunctionType && feature->long_attr("MINOR_CODE") == 199) continue;
	    if (!feature->primitive()) continue;
	    if (feature->type() == MapFeature::AreaType && feature->face_primitive()->npts()<0) continue;
	    if (feature->type() == MapFeature::LineType && feature->edge_primitive()->npts()<0) continue;
	    if (viewstate && !viewstate->themegs(feature)) continue;
	    MapFeatureComp* mfc = new MapFeatureComp(feature);
	    ((DlgFeature*)feature)->graphic(mfc);
	    Append(mfc);
	}
	for (int i = 0; i < _mapfc->nfeats(); i++) {
	    MapFeature* feature = _mapfc->feature(i);
	    if (feature->long_attr("MAJOR_CODE") == 0) continue;
	    if (feature->long_attr("MAJOR_CODE") == 40 &&
		feature->long_attr("MINOR_CODE") != 199)
		continue;
	    if (!feature->primitive()) continue;
	    if (feature->type() == MapFeature::AreaType && feature->face_primitive()->npts()<0) continue;
	    if (feature->type() == MapFeature::LineType && feature->edge_primitive()->npts()<0) continue;
	    if (viewstate && !viewstate->themegs(feature)) continue;
	    MapFeatureComp* mfc = new MapFeatureComp(feature);
	    ((DlgFeature*)feature)->graphic(mfc);
	    Append(mfc);
	}
	if (editor) {
	editor->FClassState((DlgFeatureClass*)_mapfc, true);
	}
	Notify(); //postpone?
    }
    else
	MapFClassComp::Interpret(cmd);
}

void DlgFClassComp::Uninterpret(Command* cmd) {
}

Component* DlgFClassComp::Copy() { return new DlgFClassComp((DlgFeatureClass*)_mapfc); }
ClassId DlgFClassComp::GetClassId() { return DLGFCLASS_COMP; }
boolean DlgFClassComp::IsA(ClassId id) {
    return id == DLGFCLASS_COMP || MapFClassComp::IsA(id);
}

/*****************************************************************************/

DlgFClassView::DlgFClassView(DlgFClassComp* c) : MapFClassView(c) {
}

DlgFClassView::~DlgFClassView() {
}

ClassId DlgFClassView::GetClassId() { return DLGFCLASS_VIEW; }

boolean DlgFClassView::IsA(ClassId id) {
    return id == DLGFCLASS_VIEW || MapFClassView::IsA(id);
}

void DlgFClassView::Interpret(Command* cmd) {
}

void DlgFClassView::Uninterpret(Command* cmd) {
}

/*****************************************************************************/

ParamList* DlgCovComp::_dlg_cov_params = nil;

DlgCovComp::DlgCovComp(DlgCoverage* vc) : MapCovComp((MapCoverage*)vc) {
}

DlgCovComp::DlgCovComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);

    //associate child fclasses
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
        MapFClassComp* fclcomp = (MapFClassComp*)GetComp(i);
        fclcomp->GetFeatureClass()->coverage(GetCoverage());
        }
}

ParamList* DlgCovComp::GetParamList() {
    if (!_dlg_cov_params) 
	GrowParamList(_dlg_cov_params = new ParamList());
    return _dlg_cov_params;
}

void DlgCovComp::GrowParamList(ParamList* pl) {
    pl->add_param("coverage", ParamStruct::required, &DlgCovScript::ReadName, this, &_mapcov);
    pl->add_param("fclasses", ParamStruct::required, &DlgCovScript::ReadChildren, this, this);
    MapCovComp::GrowParamList(pl);
}

DlgCovComp::~DlgCovComp() {}

MapFClassComp* DlgCovComp::CreateFClassComp(MapFeatureClass* mfc) {
    return new DlgFClassComp((DlgFeatureClass*)mfc);
}

/*****************************************************************************/

ParamList* DlgLibComp::_dlg_lib_params = nil;

DlgLibComp::DlgLibComp(DlgLibrary* vl) : MapLibComp(vl) {
}

DlgLibComp::DlgLibComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);

    //associate child coverages
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
        MapCovComp* covcomp = (MapCovComp*)GetComp(i);
        covcomp->GetCoverage()->library(GetLibrary());
        }
}

ParamList* DlgLibComp::GetParamList() {
    if (!_dlg_lib_params) 
	GrowParamList(_dlg_lib_params = new ParamList());
    return _dlg_lib_params;
}

void DlgLibComp::GrowParamList(ParamList* pl) {
    pl->add_param("library", ParamStruct::required, &DlgLibScript::ReadName, this, &_maplib);
    pl->add_param("coverages", ParamStruct::required, &DlgLibScript::ReadChildren, this, this);
    MapLibComp::GrowParamList(pl);
}

DlgLibComp::~DlgLibComp() {}

MapCovComp* DlgLibComp::CreateCovComp(MapCoverage* mc) {
    return new DlgCovComp((DlgCoverage*)mc);
}

/*****************************************************************************/

ParamList* DlgDbComp::_dlg_db_params = nil;

DlgDbComp::DlgDbComp(DlgDatabase* vdb) : MapDbComp((MapDatabase*)vdb) {
}

DlgDbComp::DlgDbComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);

    //associate child libraries
    Iterator i;
    for (First(i); !Done(i); Next(i)) {
        MapLibComp* libcomp = (MapLibComp*)GetComp(i);
        libcomp->GetLibrary()->database(GetDatabase());
        }
}

ParamList* DlgDbComp::GetParamList() {
    if (!_dlg_db_params) 
	GrowParamList(_dlg_db_params = new ParamList());
    return _dlg_db_params;
}

void DlgDbComp::GrowParamList(ParamList* pl) {
    pl->add_param("path", ParamStruct::required, &DlgDbScript::ReadPath, this, &_mapdb);
    pl->add_param("libraries", ParamStruct::required, &DlgDbScript::ReadChildren, this, this);
    MapDbComp::GrowParamList(pl);
}

DlgDbComp::~DlgDbComp() {}

MapLibComp* DlgDbComp::CreateLibComp(MapLibrary* ml) {
    return new DlgLibComp((DlgLibrary*)ml);
}

/*****************************************************************************/

ParamList* DlgIdrawComp::_dlg_idraw_params = nil;

DlgIdrawComp::DlgIdrawComp () : MapIdrawComp() { }
ClassId DlgIdrawComp::GetClassId () { return DLG_IDRAW_COMP; }

boolean DlgIdrawComp::IsA (ClassId id) {
    return DLG_IDRAW_COMP == id || MapIdrawComp::IsA(id);
}

DlgIdrawComp::DlgIdrawComp (istream& in) {
    _valid = GetParamList()->read_args(in, this);
}

ParamList* DlgIdrawComp::GetParamList() {
    if (!_dlg_idraw_params) 
	GrowParamList(_dlg_idraw_params = new ParamList());
    return _dlg_idraw_params;
}

void DlgIdrawComp::GrowParamList(ParamList* pl) {
    pl->add_param("databases", ParamStruct::required, &DlgIdrawScript::ReadChildren, this, this);
    MapIdrawComp::GrowParamList(pl); 
}

Component* DlgIdrawComp::Copy () {
    return new DlgIdrawComp();
}
