/*
 * Copyright (c) 1995 Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Unidraw/catalog.h>
#include <MapUnidraw/gsview.h>

#include <InterViews/display.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>

#include <IV-look/kit.h>
#include <IV-X11/xdisplay.h>
#include <IV-X11/xcolor.h>
#include <IV-X11/xpattern.h>
#undef None

#include <IVGlyph/figure.h>

/*****************************************************************************/

ObservableGS::ObservableGS(Graphic* gs) 
: FullGraphic(gs), Observable()
{
}

ObservableGS::~ObservableGS() {}

void ObservableGS::SetColors(PSColor* f, PSColor* b) {
    FullGraphic::SetColors(f, b);
    notify();
}

void ObservableGS::SetPattern(PSPattern* p) {
    FullGraphic::SetPattern(p);
    notify();
}

void ObservableGS::SetBrush(PSBrush* b) {
    FullGraphic::SetBrush(b);
    notify();
}

/*****************************************************************************/

BrushView::BrushView(ObservableGS* gs) 
:Patch(nil), Observer()
{
    _obs_gs = gs;
    _obs_gs->attach(this);
    update(gs);
}

BrushView::~BrushView() {
    _obs_gs->detach(this);
}

static float MENU_WIDTH = 1.3;   /* in cm */
static float MENU_HEIGHT = 0.5;

void BrushView::update(Observable*) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& wk = *WidgetKit::instance();
    PSColor* fg = _obs_gs->GetFgColor();
    PSBrush* brush = _obs_gs->GetBrush();
    Coord w = (MENU_WIDTH*ivcm);
    Coord h = (MENU_HEIGHT*ivcm);
    if (brush->None())
	body(
	    lk.overlay(
		lk.center(lk.vspace(h)),
		lk.center(lk.hspace(w)),
		lk.center(wk.label("None"))
	    )
	);
    else
	body(
	    lk.overlay(
		lk.center(lk.vspace(h)),
		lk.center(lk.hspace(w)),
		lk.center(new Fig31Line(brush, fg, nil,
			      0, 0, MENU_WIDTH*ivcm, 0))
	    )
	);
    redraw();
}

/*****************************************************************************/

PatternView::PatternView(ObservableGS* gs)
: Patch(nil), Observer() {
    _obs_gs = gs;
    _obs_gs->attach(this);
    update(gs);
}

PatternView::~PatternView() {
    _obs_gs->detach(this);
}

void PatternView::update(Observable*) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& wk = *WidgetKit::instance();
    PSColor* fg = _obs_gs->GetFgColor();
    PSColor* bg = _obs_gs->GetBgColor();
    PSPattern* pat = _obs_gs->GetPattern();
    Color * fgcolor = new Color(*fg);
    Resource::ref(fgcolor);
    fgcolor->rep(Session::instance()->default_display()->rep()->default_visual_)->stipple_ = pat->rep()->pixmap_;
    Color * bgcolor = new Color(*bg);
    Resource::ref(bgcolor);
    bgcolor->rep(Session::instance()->default_display()->rep()->default_visual_)->stipple_ = pssolid->rep()->pixmap_;
    Coord w = (MENU_WIDTH*ivcm);
    Coord h = (MENU_HEIGHT*ivcm);
    if (pat->None())
	body(
	    lk.overlay(
		lk.center(lk.vspace(h)),
		lk.center(lk.hspace(w)),
		lk.center(wk.label("None"))
	    )
	);
    else
	body(
	    lk.overlay(
		lk.center(
		    new Fig31Rectangle(psnonebr, bgcolor, bgcolor,
				       0, 0, w, h)),
		lk.center(
		    new Fig31Rectangle(psnonebr, fgcolor, fgcolor,
				       0, 0, w, h))
	    )
	);
    redraw();
    Resource::unref(bgcolor);
    Resource::unref(fgcolor);
}
