/*
 * Copyright 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <UtmUnidraw/utmeditor.h>
#include <UtmUnidraw/utmfunc.h>
#include <MapServ/maphandler.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcatalog.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapcreator.h>
#include <MapUnidraw/mapviewer.h>
#include <MapUnidraw/mapviewstate.h>
#include <Unidraw/catalog.h>
#include <Unidraw/iterator.h>
#include <Unidraw/unidraw.h>
#include <InterViews/transformer.h>
#include <ComTerp/comterp.h>
#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>

/*****************************************************************************/

UtmEditor::UtmEditor(OverlayComp* gc, OverlayKit* ok) 
: MapEditor(false, ok)
{
    Init(gc, "UtmEditor");

}

UtmEditor::UtmEditor(boolean initflag, OverlayKit* ok) : MapEditor(initflag, ok) {
}

UtmEditor::UtmEditor(const char* file, OverlayKit* ok)
: MapEditor(false, ok)
{
    if (file == nil) {
	Init();

    } else {
	Catalog* catalog = unidraw->GetCatalog();
	OverlayComp* comp;

	if (catalog->Retrieve(file, (Component*&) comp)) {
	  AttributeValue* sweav;
	  AttributeValue* swnav;
	  if ((sweav = comp->FindValue("sw_e")) && (swnav = comp->FindValue("sw_n"))) {
	    double x1 = sweav->double_val();
	    double y1 = swnav->double_val();
	    float tx1, ty1;
	    tx1 = x1; ty1 = y1;
	    
	    // use the mapviewstate mechanism
		 MapViewState* vs = GetMapViewState();
	    if (!vs->deftrans()) {
	      // set it according to the first import coming in
		   Transformer* tr = new Transformer();
	      tr->translate(-tx1, -ty1);
	      vs->settrans(tr);
	    }
	    if (!vs->implicitproj()) {
	      AttributeList* al = comp->attrlist();
	      Attribute* pa = al ? al->GetAttr("proj") : nil;
	      if (pa) {
		char* pj = (char*)pa->Value()->string_ptr();
		char* el = (char*)al->GetAttr("ellps")->Value()->string_ptr();
		int zn = al->GetAttr("zone")->Value()->int_val();
		if (pj && el) {
		  char pjs[40];
		  char els[40];
		  char zs[40];
		  sprintf(pjs, "proj=%s", pj);
		  sprintf(els, "ellps=%s", el);
		  sprintf(zs, "zone=%d", zn);
		  char* parms[4] = { pjs, els, zs, "no_defs" };
		  PJ* iproj = pj_init(4, parms);
		  vs->implicitproj(iproj);
		}
	      }
	    }
	  }
	  if (comp->IsA(MAPFCLASS_COMP)) {
	    MapIdrawComp* idcomp = new MapIdrawComp();
	    idcomp->Append(comp);
	    Init(idcomp);
	  }
	  else
	    Init(comp);
	  
	} else {
	  Init();
	  fprintf(stderr, "utmviewer: couldn't open %s\n", file);
	}
    }
}

void UtmEditor::Init (OverlayComp* comp, const char* name) {
    if (!comp) comp = new MapIdrawComp;
    _overlay_kit->Init(comp, name);
}


void UtmEditor::AddCommands(ComTerp* comterp) {

    comterp->add_command("vhclview", new VhclViewFunc(comterp, this));
    comterp->add_command("moveview", new VhclMoveToFunc(comterp, this));
    comterp->add_command("rteview", new RteViewFunc(comterp, this));

    comterp->add_command("paste", new PasteFunc(comterp, this));
    comterp->add_command("readonly", new ReadOnlyFunc(comterp, this));
    comterp->add_command("mapviewstate", new MapViewStateFunc(comterp, this));

    MapservHandler::AddCommands(comterp);
    ComEditor::AddCommands(comterp);
}

